# Hunter's Knife Arena Addon
#  
# Install instructions:
#       1. Install Mattie's EventScripts 2.1 plugin:
#           http://mattie.info/cs/
#
#       2. Copy and upload this script to:
#           <gamedir>/addons/eventscripts/knife_arena/
#
#       3. Add the following line somewhere in autoexec.cfg:
#           es_load knife_arena
#
################################################################################
#import EventScripts
import es
#import Libraries
import gamethread
import cfglib
import cmdlib
import playerlib
import weaponlib
import random
import time

# Addon Information
info = es.AddonInfo()
info.name           = "Knife Arena"
info.version        = "3.0.0"
info.author         = "Hunter"
info.url            = "http://addons.eventscripts.com/addons/user/289"
info.description    = "Turns your server into a 1vs1 Knife Arena"
info.basename       = "knife_arena"

hunter_knife_arena_ver   = info.version
hunter_knife_arena_text  = 'Hunters '+info.name+', www.sourceplugins.de, '+info.version+', ES 2.1.1.338+'

# Server Variables
knife_arena_config       = cfglib.AddonCFG('%s/knife_arena.cfg' % es.getAddonPath('knife_arena'))
knife_arena_limit        = knife_arena_config.cvar('knife_arena_limit', 0, 'Is there a limit on how often a player can respawn? ( 0 = off | >0 = limit )')
knife_arena_timelimit    = knife_arena_config.cvar('knife_arena_timelimit', 30, 'Whats the time limit of 1vs1 fights?')
knife_arena_health       = knife_arena_config.cvar('knife_arena_health', 100, 'The start health of players in a fight')
knife_arena_config.write()

# Helper Classes
class Fight(object):
    def __init__(self, players):
        self.time = int(knife_arena_timelimit)
        self.area = None
        self.players = players
        self.running = False
        for player in self.players:
            player.set_fight(self)
            es.centertell(player.userid, 'Found opponent...')
        gamethread.delayedname(1, repr(self), self.prepare)
        
    def __del__(self):
        gamethread.cancelDelayed(repr(self))
        
    def prepare(self):
        self.running = True
        self.area = knife_arena_areas.random_free_area()
        if self.area:
            self.area.set_fight(self)
            for player in self.players:
                es.entitysetvalue(player.index, 'origin', self.area.get_origin(player.teamid))
               #es.server.queuecmd('es_xsetpos %d %s' % (player.userid, self.area.get_origin(player.teamid)))
               #es.entitysetvalue(player.index, 'angles', self.area.get_angles(player.teamid))
                es.server.queuecmd('es_xsetang %d %s' % (player.userid, self.area.get_angles(player.teamid)))
                es.centertell(player.userid, 'Prepare to Fight...')
                player.freeze(1)
            gamethread.delayedname(1, repr(self), self.goodluck)
        else:
            gamethread.delayedname(3, repr(self), self.prepare)
        
    def goodluck(self):
        if self.running:
            for player in self.players:
                es.centertell(player.userid, 'Good Luck!')
                es.server.queuecmd('es_xgive %d weapon_knife' % player.userid)
            gamethread.delayedname(1, repr(self), self.start)
        
    def start(self):
        if self.running:
            for player in self.players:
                es.playsound(player.userid, 'ambient/misc/brass_bell_c.wav', 0.2)
                player.freeze(0)
            gamethread.delayedname(1, repr(self), self.fight)
        
    def fight(self):
        if self.running:
            self.time -= 1
            for player in self.players:
                player.set('cash', '0')
            if self.time > 0:
                for player in self.players:
                    es.centertell(player.userid, '%d/%d remaining' % (self.time, int(knife_arena_timelimit)))
            elif int(knife_arena_timelimit) > 0:
                for player in self.players:
                    es.centertell(player.userid, 'Timeout! Searching another opponent...')
                self.end()
            gamethread.delayedname(1, repr(self), self.fight)

    def end(self):
        self.running = False
        for player in self.players:
            player.spawn()
        if self.area:
            gamethread.delayedname(5, repr(self), self.area.reset)
        
    def loose(self, looser):
        if looser in self.players:
            self.players.remove(looser)
        if self.running:
            winner = self.players.pop()
            es.centertell(looser.userid, 'Lost! Respawning soon...')
            es.centertell(winner.userid, 'Win! Searching new opponent...')
            gamethread.delayedname(3, repr(self), winner.spawn)
        self.end()

class Area(object):
    def __init__(self, teamid, index):
        self.fight = None
        self.corners = {}
        self.set_corner(teamid, index)
    
    def set_corner(self, teamid, index):
        self.corners[teamid] = {}
        self.corners[teamid]['origin'] = es.entitygetvalue(index, 'origin')
        self.corners[teamid]['angles'] = es.entitygetvalue(index, 'angles')
        
    def has_corners(self):
        return len(self.corners) == 2
        
    def has_fight(self):
        return self.fight != None

    def set_fight(self, fight):
        self.fight = fight
        
    def get_origin(self, teamid):
        return self.corners[teamid]['origin']
        
    def get_angles(self, teamid):
        return self.corners[teamid]['angles']

    def reset(self):
        self.fight = None

class AreaManager(object):
    def __init__(self):
        self.areas = {}
        
    def search_areas(self):
        self.areas = {}
        targets = es.createentitylist('info_target')
        for index in targets:
            self.add_area_point(index)
    
    def add_area_point(self, index):
        targetname = es.entitygetvalue(index, 'targetname')
        if targetname.endswith('_ka_t'):
            self.set_area_corner(int(targetname.replace('_ka_t', '')), 2, index)
        elif targetname.endswith('_ka_ct'):
            self.set_area_corner(int(targetname.replace('_ka_ct', '')), 3, index)

    def set_area_corner(self, areaid, teamid, index):
        if areaid in self.areas:
            self.areas[areaid].set_corner(teamid, index)
        else:
            self.areas[areaid] = Area(teamid, index)
    
    def check_areas(self):
        if not self.areas:
            return None
        
        for areaid in self.areas:
            if not self.areas[areaid].has_corners():
                return False
                
        return True
        
    def reset_areas(self):
        for areaid in self.areas:
            self.areas[areaid].reset()
            
    def random_free_area(self):
        free_areas = []
        for areaid in self.areas:
            if self.areas[areaid].has_corners() and not self.areas[areaid].has_fight():
                free_areas.append(self.areas[areaid])
        
        return random.choice(free_areas)

class Player(object):
    def __init__(self, userid):
        self.userid = userid
        self.player = playerlib.getPlayer(self.userid)
        self.origin = es.entitygetvalue(self.index, 'origin')
        self.angles = es.entitygetvalue(self.index, 'angles')
        self.lives = int(knife_arena_limit)
        self.fight = None
        
    def __del__(self):
        gamethread.cancelDelayed(repr(self))
        
    def __int__(self):
        return self.userid
        
    def __getattr__(self, attr):
        if attr == 'teamid':
            return es.getplayerteam(self.userid)
        return getattr(self.player, attr)
        
    def set_fight(self, fight):
        self.fight = fight
        
    def exists(self):
        return es.exists('userid', self.userid)
        
    def refresh_location(self):
        if self.exists():
            self.origin = es.entitygetvalue(self.index, 'origin')
            self.angles = es.entitygetvalue(self.index, 'angles')
        
    def reset(self):
        if self.fight:
            self.fight.end()
        self.fight = None
        self.lives = int(knife_arena_limit)
        
    def strip(self):
        if self.exists():
            handle = es.getplayerhandle(self.userid)
            for index in weaponlib.getIndexList('#all'):
                if es.getindexprop(index, 'CBaseEntity.m_hOwnerEntity') == handle:
                    es.server.queuecmd('es_xremove %d' % index)
        
    def spawn(self, real=False):
        if self.exists():
            self.strip()
            self.set('cash', 0)
            self.set('health', int(knife_arena_health))
            if not real:
                es.entitysetvalue(self.index, 'origin', self.origin)
               #es.server.queuecmd('es_xsetpos %d %s' % (self.userid,  self.origin)
               #es.entitysetvalue(self.index, 'angles', self.angles)
                es.server.queuecmd('es_xsetang %d %s' % (self.userid, self.angles))
            else:
                self.refresh_location()
        if self.fight:
            self.fight.loose(self)
            self.fight = None
        knife_arena_queues.enter(self)
        
    def death(self, real=False):
        if self.isdead or real:
            if self.lives:
                self.lives -= 1
            if self.fight:
                self.fight.loose(self)
                self.fight = None
            if self.lives > 0 or int(knife_arena_limit) == 0:
                gamethread.delayedname(5, repr(self), self.respawn)
            knife_arena_queues.leave(self)
            
    def respawn(self):
        if self.exists():
            es.server.queuecmd('es_xsetplayerprop %d "CCSPlayer.m_iPlayerState" 0' % self.userid)
            es.server.queuecmd('es_xsetplayerprop %d "CCSPlayer.baseclass.m_lifeState" 512' % self.userid)
            es.server.queuecmd('es_xspawnplayer %d' % self.userid)
            
    def disconnect(self):
        if self.fight:
            self.fight.loose(self)
            self.fight = None
        knife_arena_queues.leave(self)
        
class PlayerManager(object):
    def __init__(self):
        self.players = {}
        self.search_players()
    
    def search_players(self):
        for userid in es.getUseridList():
            if not userid in self.players:
                self.players[userid] = Player(userid)
        
        delete_users = []
        for userid in self.players:
            if not es.exists('userid', userid):
                delete_users.append(userid)
                
        for userid in delete_users:
            del self.players[userid]

    def reset_players(self):
        for userid in self.players:
            self.players[userid].reset()
            
    def get_player(self, userid):
        if not userid in self.players:
            self.players[userid] = Player(userid)
        
        return self.players[userid]

class Queue(object):
    def __init__(self):
        self.queue = []
        
    def enter(self, player):
        if not player in self.queue:
            self.queue.insert(0, player)
        
    def leave(self, player=None):
        if player:
            return self.queue.remove(player) if player in self.queue else None
        else:
            random.shuffle(self.queue)
            player = self.queue.pop()
            return player
        
    def has_players(self):
        return len(self.queue) > 0

class QueueManager(object):
    def __init__(self):
        self.queues = {2: Queue(), 3: Queue()}
        
    def __del__(self):
        gamethread.cancelDelayed(repr(self))
        
    def enter(self, player):
        if player.teamid > 1:
            es.centertell(player.userid, 'Searching new opponent...')
            self.queues[player.teamid].enter(player)

    def leave(self, teamid, player=None):
        if teamid in self.queues:
            if self.queues[teamid].has_players():
                return self.queues[teamid].leave(player)
                
    def check(self):
        gamethread.delayedname(3, repr(self), self.check)
        if self.queues[2].has_players() and self.queues[3].has_players():
            Fight([self.queues[2].leave(), self.queues[3].leave()])
            
    def watch_queues(self):
        gamethread.delayedname(1, repr(self), self.check)

# Global Variables
knife_arena_areas        = AreaManager()
knife_arena_players      = PlayerManager()
knife_arena_queues       = QueueManager()
knife_arena_restart      = False

def load():
    public = es.ServerVar('hu_ka', info.version, info.name)
    public.makepublic()
    
    cmdlib.registerSayCommand('!watch', _watch_cmd, 'Watch')
    cmdlib.registerSayCommand('!fight', _fight_cmd, 'Fight')
    
    knife_arena_config.execute()
    knife_arena_areas.search_areas()
    knife_arena_players.search_players()

    es.log(hunter_knife_arena_text)
    es.msg('#multi', '#green[KnifeArena] #defaultLoaded')
    
def unload():
    cmdlib.unregisterSayCommand('!watch')
    cmdlib.unregisterSayCommand('!fight')
    
    es.msg('#multi', '#green[KnifeArena] #defaultUnloaded')
    
def es_map_start(event_var):
    global knife_arena_restart
    knife_arena_restart = True
    knife_arena_players.search_players()

def es_client_command(event_var):
    if (str(event_var['command']) == '!hunter_knife_arena_ver') or (str(event_var['command']) == '!hunter_all_ver'):
        es.cexec(int(event_var['userid']), 'echo '+hunter_knife_arena_text)

def player_activate(event_var):
    knife_arena_players.search_players()
    gamethread.delayed(30, es.cexec, (int(event_var['userid']), 'echo '+hunter_knife_arena_text))
    if knife_arena_areas.check_areas():
        gamethread.delayed(15, es.tell, (int(event_var['userid']), '#multi', '#green[KnifeArena] #defaultThis server is running a knife-only area'))
    
def round_start(event_var):
    global knife_arena_restart
    if knife_arena_restart:
        knife_arena_restart = False
        knife_arena_areas.search_areas()
        status = knife_arena_areas.check_areas()
        if status:
            es.msg('#multi', '#green[KnifeArena] #lightgreenFound areas!')
        elif status == False:
            es.msg('#multi', '#green[KnifeArena] #lightgreenThe areas on this map are screwed up!')
        else:
            es.msg('#multi', '#green[KnifeArena] #lightgreenNo areas were found on this map!')
            
    knife_arena_areas.reset_areas()
    knife_arena_players.reset_players()

def round_freeze_end(event_var):
    if knife_arena_areas.check_areas():
        knife_arena_queues.watch_queues()

def player_spawn(event_var):
    if knife_arena_areas.check_areas():
        player = knife_arena_players.get_player(int(event_var['userid']))
        player.spawn(True)
        es.server.queuecmd('es_xfire %s cs_ragdoll kill' % event_var['userid'])
        for index in weaponlib.getIndexList('#all'):
            if es.getindexprop(index, 'CBaseEntity.m_hOwnerEntity') == -1:
                es.server.queuecmd('es_xremove %d' % index)

def player_death(event_var):
    if knife_arena_areas.check_areas():
        player = knife_arena_players.get_player(int(event_var['userid']))
        player.death(True)

def player_disconnect(event_var):
    if knife_arena_areas.check_areas():
        player = knife_arena_players.get_player(int(event_var['userid']))
        player.death(True)

def player_team(event_var):
    if knife_arena_areas.check_areas():
        player = knife_arena_players.get_player(int(event_var['userid']))
        if int(event_var['disconnect']) == 0:
            player.death()
        else:
            player.disconnect()

def _watch_cmd(userid, args):
    if knife_arena_areas.check_areas():
        es.server.queuecmd('es_xfire %d !self setteam 1' % userid)
        
def _fight_cmd(userid, args):
    if knife_arena_areas.check_areas():
        player = knife_arena_players.get_player(userid)
        if player.teamid < 2:
            if len(playerlib.getUseridList('#t')) < len(playerlib.getUseridList('#ct')):
                es.server.queuecmd('es_xfire %d !self setteam 2' % player)
            else:
                es.server.queuecmd('es_xfire %d !self setteam 3' % player)
        if player.isdead == 1:
            player.death()
